==> fxt-2009.08.05.tgz: fxt/src/sort/bsearchptrfunc.h <==
#if !defined HAVE_BSEARCHPTRFUNC_H__
#define      HAVE_BSEARCHPTRFUNC_H__


#include "fxttypes.h"

template <typename Type>
ulong ptr_bsearch(/*const Type *f,*/ ulong n, Type const*const*x, const Type v,
                  int (*cmp)(const Type &, const Type &))
// Return index of ptr i to first element in f[] that is == v
// i.e. *x[i] == v, with i minimal.
// Return ~0 if there is no such element.
// x[] must be (ptr-)sorted in ascending order:
// *x[i] <= *x[i+i]
// Must have  n!=0
{
    ulong nlo=0, nhi=n-1;
    while ( nlo != nhi )
    {
        ulong t = (nhi+nlo)/2;

//        if ( *x[t] < v )  nlo = t + 1;
        if ( cmp(*x[t],v)<0 )  nlo = t + 1;
        else                   nhi = t;
    }

//    if ( *x[nhi]==v )  return nhi;
    if ( cmp(*x[nhi],v)==0 )  return nhi;
    else                      return ~0UL;
}
// -------------------------


template <typename Type>
ulong ptr_bsearch_geq(/*const Type *f,*/ ulong n, Type const*const*x, const Type v,
                      int (*cmp)(const Type &, const Type &))
// Return index of ptr of first element in f[] that is >= v
// i.e. *x[i] >= v, with i minimal.
// Return ~0 if there is no such element
// x[] must be (ptr-)sorted in ascending order:
// *x[i] <= *x[i+i]
// Must have  n!=0
{
    ulong nlo=0, nhi=n-1;
    while ( nlo != nhi )
    {
        ulong t = (nhi+nlo)/2;

//        if ( x[t] < v )  nlo = t + 1;
        if ( cmp(x[t],v)<0 )  nlo = t + 1;
        else                  nhi = t;
    }

//    if ( *x[nhi]>=v )  return nhi;
    if ( cmp(*x[nhi],v)>=0 )  return nhi;
    else                      return ~0UL;
}
// -------------------------


#endif // !defined HAVE_BSEARCHPTRFUNC_H__

==> fxt-2010.05.04.tgz: fxt/src/sort/bsearchptrfunc.h <==
#if !defined HAVE_BSEARCHPTRFUNC_H__
#define      HAVE_BSEARCHPTRFUNC_H__


#include "fxttypes.h"

template <typename Type>
ulong ptr_bsearch(/*const Type *f,*/ ulong n, Type const*const*x, const Type v,
                  int (*cmp)(const Type &, const Type &))
// Return index of ptr i to first element in f[] that is == v
// i.e. *x[i] == v, with i minimal.
// Return n if there is no such element.
// x[] must be (ptr-)sorted in ascending order:
// *x[i] <= *x[i+i]
// Must have  n!=0
{
    ulong nlo=0, nhi=n-1;
    while ( nlo != nhi )
    {
        ulong t = (nhi+nlo)/2;

//        if ( *x[t] < v )  nlo = t + 1;
        if ( cmp(*x[t],v)<0 )  nlo = t + 1;
        else                   nhi = t;
    }

//    if ( *x[nhi]==v )  return nhi;
    if ( cmp(*x[nhi],v)==0 )  return nhi;
    else                      return n;
}
// -------------------------


template <typename Type>
ulong ptr_bsearch_geq(/*const Type *f,*/ ulong n, Type const*const*x, const Type v,
                      int (*cmp)(const Type &, const Type &))
// Return index of ptr of first element in f[] that is >= v
// i.e. *x[i] >= v, with i minimal.
// Return n if there is no such element
// x[] must be (ptr-)sorted in ascending order:
// *x[i] <= *x[i+i]
// Must have  n!=0
{
    ulong nlo=0, nhi=n-1;
    while ( nlo != nhi )
    {
        ulong t = (nhi+nlo)/2;

//        if ( x[t] < v )  nlo = t + 1;
        if ( cmp(x[t],v)<0 )  nlo = t + 1;
        else                  nhi = t;
    }

//    if ( *x[nhi]>=v )  return nhi;
    if ( cmp(*x[nhi],v)>=0 )  return nhi;
    else                      return n;
}
// -------------------------


#endif // !defined HAVE_BSEARCHPTRFUNC_H__
